package net.jieyuu.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.enums.BizCodeEnum;
import net.jieyuu.enums.CouponCategoryEnum;
import net.jieyuu.enums.CouponPublishEnum;
import net.jieyuu.enums.CouponStateEnum;
import net.jieyuu.exception.BizException;
import net.jieyuu.interceptor.LoginInterceptor;
import net.jieyuu.mapper.CouponRecordMapper;
import net.jieyuu.model.CouponDO;
import net.jieyuu.mapper.CouponMapper;
import net.jieyuu.model.CouponRecordDO;
import net.jieyuu.model.LoginUser;
import net.jieyuu.request.NewUserRequest;
import net.jieyuu.service.CouponService;
import net.jieyuu.utils.CommonUtil;
import net.jieyuu.utils.JsonData;
import net.jieyuu.vo.CouponVO;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jieyuu
 * @since 2024-07-15
 */
@Service
@Slf4j

public class CouponServiceImpl implements CouponService {

    @Autowired
    private CouponMapper couponMapper;

    @Autowired
    private CouponRecordMapper couponRecordMapper;

    @Autowired
    private RedissonClient redissonClient;

    @Override
    public Map<String, Object> pageCouponActivity(int page, int size) {
        Page<CouponDO> pageInfo = new Page<>(page, size);

        IPage<CouponDO> couponDOIPage = couponMapper.selectPage(pageInfo, new QueryWrapper<CouponDO>()
                .eq("publish", CouponPublishEnum.PUBLISH)
                .eq("category", CouponCategoryEnum.PROMOTION)
                .orderByAsc("create_time"));

        Map<String, Object> pageMap = new HashMap<>(3);

        pageMap.put("total_record", couponDOIPage.getTotal());
        pageMap.put("total_page", couponDOIPage.getPages());
        pageMap.put("current_data", couponDOIPage.getRecords().stream().map(obj -> beanProcess(obj)).collect(Collectors.toList()));

        return pageMap;
    }

    /**
     * 领券接口
     * 1、获取优惠券是否纯在
     * 2、校验优惠券是否可以领取: 时间 库存 超过限制
     * 3、扣减库存
     * 4、保存领券记录
     *
     * @param couponId
     * @param category
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public JsonData addCoupon(long couponId, CouponCategoryEnum category) {
        LoginUser loginUser = LoginInterceptor.threadLocal.get();

        String lockKey = "lock:coupon:" + couponId;
        RLock rlock = redissonClient.getLock(lockKey);
        // 多个线程进入 会等待锁
        rlock.lock();
        log.info("领券接口加锁成功:{}", Thread.currentThread().getId());

        try {
            // 执行业务逻辑
            CouponDO couponDO = couponMapper.selectOne(new QueryWrapper<CouponDO>()
                    .eq("id", couponId)
                    .eq("category", category.name()));
            //优惠券是否可以领取
            this.couponCheck(couponDO, loginUser.getId());

            //构建领券记录
            CouponRecordDO couponRecordDO = new CouponRecordDO();
            BeanUtils.copyProperties(couponDO, couponRecordDO);
            couponRecordDO.setCreateTime(new Date());
            couponRecordDO.setUseState(CouponStateEnum.NEW.name());
            couponRecordDO.setCouponId(couponId);
            couponRecordDO.setUserName(loginUser.getName());
            couponRecordDO.setUserId(loginUser.getId());
            couponRecordDO.setId(null);

            //扣减库存
            int rows = couponMapper.reduceStock(couponId);
            if (rows == 1) {
                //扣减库存成功才保存记录
                couponRecordMapper.insert(couponRecordDO);
            } else {
                log.warn("发放优惠券错误:{},用户:{}", couponDO, loginUser);
                throw new BizException(BizCodeEnum.COUPON_NO_STOCK);
            }
        } finally {
            // 释放锁
            rlock.unlock();
            log.info("解锁成功");
        }
        return JsonData.buildSuccess();
    }

    /**
     * 微服务调用时没登录，没有token
     * <p>
     * 本地发放优惠券，需要构造token存入threadlocal
     *
     * @param newUserRequest
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public JsonData initNewUserCoupon(NewUserRequest newUserRequest) {
        LoginUser loginUser = new LoginUser();
        loginUser.setName(newUserRequest.getName());
        loginUser.setId(newUserRequest.getUserId());
        // 发放优惠券接口需要threadlocal
        LoginInterceptor.threadLocal.set(loginUser);

        List<CouponDO> couponList = couponMapper.selectList(new QueryWrapper<CouponDO>()
                .eq("category", CouponCategoryEnum.NEW_USER.name()));
        for (CouponDO couponDO : couponList) {
            // 幂等操作 调用需要加锁
            // 发放
            this.addCoupon(couponDO.getId(), CouponCategoryEnum.NEW_USER);
        }
        return JsonData.buildSuccess();
    }


    /**
     * 校验优惠券是否可以领取
     *
     * @param couponDO
     * @param id
     */
    private void couponCheck(CouponDO couponDO, Long id) {

        if (couponDO == null) {
            throw new BizException(BizCodeEnum.COUPON_NO_EXITS);
        }

        //库存不足
        if (couponDO.getStock() <= 0) {
            throw new BizException(BizCodeEnum.COUPON_NO_STOCK);
        }

        //未经发布优惠券 确认非法
        if (!couponDO.getPublish().equals(CouponPublishEnum.PUBLISH.name())) {
            throw new BizException(BizCodeEnum.COUPON_GET_FAIL);
        }

        //判断是否在领取时间
        long time = CommonUtil.getCurrentTimestamp();
        long start = couponDO.getStartTime().getTime();
        long end = couponDO.getEndTime().getTime();
        if (time < start || time > end) {
            throw new BizException(BizCodeEnum.COUPON_OUT_OF_TIME);
        }

        //用户是否超过限制
        //查询用户已经拥有的优惠券数量
        int recordNum = couponRecordMapper.selectCount(new QueryWrapper<CouponRecordDO>()
                .eq("coupon_id", couponDO.getId())
                .eq("user_id", id));

        //判断优惠券是否超过限制
        if (recordNum >= couponDO.getUserLimit()) {
            throw new BizException(BizCodeEnum.COUPON_OUT_OF_LIMIT);
        }
    }


    private CouponVO beanProcess(CouponDO couponDO) {

        CouponVO couponVO = new CouponVO();
        BeanUtils.copyProperties(couponDO, couponVO);

        return couponVO;
    }

}
