package net.jieyuu.service.impl;


import lombok.extern.slf4j.Slf4j;
import net.jieyuu.component.MailService;
import net.jieyuu.constant.CacheKey;
import net.jieyuu.enums.BizCodeEnum;
import net.jieyuu.enums.SendCodeEnum;
import net.jieyuu.service.NotifyService;
import net.jieyuu.utils.CheckUtil;
import net.jieyuu.utils.CommonUtil;
import net.jieyuu.utils.JsonData;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class NotifyServiceImpl implements NotifyService {

    @Autowired
    private MailService mailService;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 验证码标题
     */
    private static final String SUBJECT = "验证码";

    /**
     * 验证码格式
     * todo 对60s的思考
     */
    private static final String CONTENT = "您的验证码是%s，有效时间为10分钟";

    /**
     * 十分钟有效
     */
    private static final int CODE_EXPIRED = 60 * 1000 * 10;


    /**
     * 前置判断：判断是否重复发送
     * 1 存储验证码到缓存
     * <p>
     * 2 发送验证码
     * 后置 存储发送记录
     *
     * @param sendCodeType
     * @param to
     * @return
     */
    @Override
    public JsonData sendCode(SendCodeEnum sendCodeType, String to) {

        String cacheKey = String.format(CacheKey.CHECK_CODE_KEY, SendCodeEnum.USER_REGISTER.name(), to);
        String cacheValue = (String) redisTemplate.opsForValue().get(cacheKey);

        //判断是否为空
        //然后判断60秒内是否重复
        if (StringUtils.isNotBlank(cacheValue)) {
            long ttl = Long.parseLong(cacheValue.split("_")[1]);
            // 判断 当前时间戳-获取时间戳 是否大于60s
            if (CommonUtil.getCurrentTimestamp() - ttl < 1000 * 60) {
                log.info("重复发送验证码,时间间隔:{} 秒", (CommonUtil.getCurrentTimestamp() - ttl) / 1000);
                return JsonData.buildResult(BizCodeEnum.CODE_LIMITED);
            }
        }

        //拼接验证码 2322_时间戳
        long currentTimestamp = CommonUtil.getCurrentTimestamp();
        //生成验证码
        String code = CommonUtil.getRandomCode(6);
        String value = code + "_" + currentTimestamp;
        redisTemplate.opsForValue().set(cacheKey, value, CODE_EXPIRED, TimeUnit.MILLISECONDS);
        if (CheckUtil.isEmail(to)) {
            //邮箱验证码
            mailService.sendMail(to, SUBJECT, String.format(CONTENT, code));
            return JsonData.buildSuccess();
        } else if (CheckUtil.isPhone(to)) {
            //短信验证码

        }

        return JsonData.buildResult(BizCodeEnum.CODE_TO_ERROR);
    }

    /**
     * 检验验证码
     *
     * @param sendCodeEnum
     * @param to
     * @param code
     * @return
     */
    @Override
    public boolean checkCode(SendCodeEnum sendCodeEnum, String to, String code) {
        //读取缓存
        String cacheKey = String.format(CacheKey.CHECK_CODE_KEY, sendCodeEnum.name(), to);
        String cacheValue = (String) redisTemplate.opsForValue().get(cacheKey);

        if (StringUtils.isNotBlank(cacheValue)) {
            //截取
            String cacheCode = cacheValue.split("_")[0];
            //判断
            if (cacheCode.equals(code)) {
                //删除验证码
                redisTemplate.delete(cacheKey);
                return true;
            }

        }
        return false;
    }
}
